-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.AddBarEditor = {}
XMacros.AddBarEditor.name = "XMacros.AddBarEditor"
XMacros.AddBarEditor.ShowAlways = "Always"
XMacros.AddBarEditor.ShowInCombat = "Only when in combat"
XMacros.AddBarEditor.ShowOutCombat = "Only when out of combat"
XMacros.AddBarEditor.ShowMouse = "Show when mouse enters bar"
XMacros.AddBarEditor.ShowInCombatMouse = "Only when in combat and mouse in bar"
XMacros.AddBarEditor.ShowOutCombatMouse = "Only when out of combat and mouse in bar"

--
-- Main functions
--

--
-- UI functions
--


function XMacros.AddBarEditor.Create(parentFrame, positionFrame)
	local barEditor = {}
	barEditor.parentFrame = parentFrame
	
	barEditor.width = 430
	barEditor.height = 280
	barEditor.windowFrame = UI.CreateFrame("SimpleWindow", "BarEditor", parentFrame)
	barEditor.windowFrame:SetVisible(false)
	barEditor.windowFrame:SetSecureMode("normal")
	barEditor.windowFrame:SetCloseButtonVisible(true)
	barEditor.windowFrame:SetTitle("Bar Editor")
	barEditor.windowFrame:SetWidth(barEditor.width)
	barEditor.windowFrame:SetHeight(barEditor.height)
	barEditor.windowFrame:SetPoint("CENTER", positionFrame, "CENTER")
	barEditor.windowFrame:SetLayer(positionFrame:GetLayer() + 1)
	barEditor.windowFrame.Event.Close = function() XMacros.AddBarEditor.WindowClose(barEditor) end
	barEditor.x = barEditor.windowFrame:GetLeft()
	barEditor.y = barEditor.windowFrame:GetTop()
	
	local labelWidth = 60
	local textWidth = 30
	local sliderWidth = 100
	local barFrame = UI.CreateFrame("Text", "RowLabelFrame", barEditor.windowFrame)
	barFrame:SetPoint("TOPLEFT", barEditor.windowFrame, "TOPLEFT", 40, 60)
	barFrame:SetText("Rows")
	barFrame:SetWidth(labelWidth)
	
	barEditor.rowCountText = UI.CreateFrame("SimpleSlider", "RowCountText", barEditor.windowFrame)
	barEditor.rowCountText:SetPoint("TOPLEFT", barFrame, "TOPRIGHT", 20, 0)
	barEditor.rowCountText:SetRange(1, 16)
	barEditor.rowCountText:SetWidth(sliderWidth)
		
	local textFrame = UI.CreateFrame("Text", "ColLabelFrame", barEditor.windowFrame)
	textFrame:SetPoint("TOPLEFT", barFrame, "TOPLEFT", 0, 40)
	textFrame:SetText("Columns")
	textFrame:SetWidth(labelWidth)
	
	barEditor.colCountText = UI.CreateFrame("SimpleSlider", "ColCountText", barEditor.windowFrame)
	barEditor.colCountText:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 20, 0)
	barEditor.colCountText:SetRange(1, 16)
	barEditor.colCountText:SetWidth(sliderWidth)
		
	local textFrame = UI.CreateFrame("Text", "IconLabelFrame", barEditor.windowFrame)
	textFrame:SetPoint("TOPLEFT", barFrame, "TOPLEFT", 0, 80)
	textFrame:SetText("Button Size")
	textFrame:SetWidth(labelWidth)
	
	barEditor.iconSizeText = UI.CreateFrame("SimpleSlider", "IconSizeText", barEditor.windowFrame)
	barEditor.iconSizeText:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 20, 0)
	barEditor.iconSizeText:SetRange(16, 256)
	barEditor.iconSizeText:SetPosition(52)
	barEditor.iconSizeText:SetWidth(sliderWidth * 2)
	
	local textFrame = UI.CreateFrame("Text", "InCombatLabelFrame", barEditor.windowFrame)
	textFrame:SetPoint("TOPLEFT", barFrame, "TOPLEFT", 0, 120)
	textFrame:SetText("Show bar")
	textFrame:SetWidth(labelWidth)
	
	barEditor.showText = UI.CreateFrame("Text", "ShowText", barEditor.windowFrame)
	barEditor.showText:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 20, 0)
	barEditor.showText:SetText(XMacros.AddBarEditor.ShowAlways)
	barEditor.showText:SetWidth(140)
		
	barEditor.showButton = UI.CreateFrame("RiftButton", "ShowButton", barEditor.windowFrame)
	barEditor.showButton:SetPoint("TOPLEFT", barEditor.showText, "TOPRIGHT", 10, -8)
	barEditor.showButton:SetText("Choose")
	barEditor.showButton.Event.LeftPress = function() XMacros.AddBarEditor.ShowOption(barEditor) end
	
	barEditor.okButton = UI.CreateFrame("RiftButton", "OKButton", barEditor.windowFrame)
	barEditor.okButton:SetPoint("BOTTOMLEFT", barEditor.windowFrame, "BOTTOMLEFT", 20, -20)
	barEditor.okButton:SetText("OK")
	barEditor.okButton.Event.LeftPress = function() XMacros.AddBarEditor.WindowClose(barEditor, true) end
	
	barEditor.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", barEditor.windowFrame)
	barEditor.cancelButton:SetPoint("BOTTOMRIGHT", barEditor.windowFrame, "BOTTOMRIGHT", -20, -20)
	barEditor.cancelButton:SetText("Cancel")
	barEditor.cancelButton.Event.LeftPress = function() XMacros.AddBarEditor.WindowClose(barEditor) end
	
	barEditor.txtPicker = XMacros.TextPicker.Create(barEditor.parentFrame, positionFrame)
	barEditor.wrnWindow = XMacros.WarningWindow.CreateWarningWindow(barEditor.parentFrame, positionFrame)

	return barEditor
end

function XMacros.AddBarEditor.WindowClose(barEditor, okPressed)
	if okPressed == true then
		local message = ""
		local rowCount = tonumber(barEditor.rowCountText:GetPosition())
		if not rowCount then
			message = message .. "row count must be a number\n"
		elseif rowCount < 1 or rowCount > 32 then
			message = message .. "row count must be in the range 1 - 32\n"
		end
		local colCount = tonumber(barEditor.colCountText:GetPosition())
		if not colCount then
			message = message .. "column count must be a number\n"
		elseif colCount < 1 or colCount > 32 then
			message = message .. "column count must be in the range 1 - 32\n"
		end
		local iconSize = tonumber(barEditor.iconSizeText:GetPosition())
		if not iconSize then
			message = message .. "button size must be a number\n"
		elseif iconSize < 16 or iconSize > 128 then
			message = message .. "button size must be in the range 16 - 128\n"
		end

		if string.len(message) > 0 then
			XMacros.WarningWindow.ShowWarningWindow(barEditor.wrnWindow, message)
		else
			local combatShow = XMacros.ButtonSets.ShowTypeAlways
			local showOption = barEditor.showText:GetText()
			if showOption == XMacros.AddBarEditor.ShowInCombat then
				combatShow = XMacros.ButtonSets.ShowTypeInCombat
			elseif showOption == XMacros.AddBarEditor.ShowOutCombat then
				combatShow = XMacros.ButtonSets.ShowTypeOutCombat
			elseif showOption == XMacros.AddBarEditor.ShowMouse then
				combatShow = XMacros.ButtonSets.ShowTypeMouse
			elseif showOption == XMacros.AddBarEditor.ShowInCombatMouse then
				combatShow = XMacros.ButtonSets.ShowTypeInCombatMouse
			elseif showOption == XMacros.AddBarEditor.ShowOutCombatMouse then
				combatShow = XMacros.ButtonSets.ShowTypeOutCombatMouse
			end
			
			barEditor.windowFrame:SetVisible(false)
			if barEditor.closeCallback then
				barEditor.closeCallback(barEditor, rowCount, colCount, iconSize, combatShow)
			end
		end
	else
		barEditor.windowFrame:SetVisible(false)
		if barEditor.closeCallback then
			barEditor.closeCallback()
		end
	end
end

function XMacros.AddBarEditor.Show(barEditor, closeCallback)
	barEditor.closeCallback = closeCallback
	barEditor.iconSizeText:SetPosition(52)
	barEditor.colCountText:SetPosition(1)
	barEditor.rowCountText:SetPosition(1)
	barEditor.showText:SetText(XMacros.AddBarEditor.ShowAlways)
	barEditor.windowFrame:SetVisible(true)
	barEditor.x = barEditor.windowFrame:GetLeft()
	barEditor.y = barEditor.windowFrame:GetTop()
end

function XMacros.AddBarEditor.SetEnabled(barEditor, enabled)
	barEditor.showButton:SetEnabled(enabled)
	barEditor.okButton:SetEnabled(enabled)
	barEditor.cancelButton:SetEnabled(enabled)
end

function XMacros.AddBarEditor.ShowOptionChosen(barEditor, showOption)
	XMacros.AddBarEditor.SetEnabled(barEditor, true)
	if showOption then
		barEditor.showText:SetText(showOption)
	end
end

function XMacros.AddBarEditor.ShowOption(barEditor)
	XMacros.AddBarEditor.SetEnabled(barEditor, false)
	local options = { XMacros.AddBarEditor.ShowAlways, XMacros.AddBarEditor.ShowInCombat, XMacros.AddBarEditor.ShowOutCombat,
		XMacros.AddBarEditor.ShowMouse, XMacros.AddBarEditor.ShowInCombatMouse, XMacros.AddBarEditor.ShowOutCombatMouse }
	XMacros.TextPicker.Show(barEditor.txtPicker, "Choose Show Option", options, function(a, showOption) XMacros.AddBarEditor.ShowOptionChosen(barEditor, showOption) end)
end
