-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.Button = {}
XMacros.Button.buttonNames = { "Left", "Middle", "Right", "Mouse4", "Mouse5" }
XMacros.Button.buttonAbbrev = { "L", "M", "R", "4", "5" }

local function AbilityDetail(abilityID)
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.Detail(abilityID)
	else
		return Inspect.Ability.Detail(abilityID)
	end
end

local function AbilityList()
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.List()
	else
		return Inspect.Ability.List()
	end
end


function XMacros.Button.Create(parentFrame, name, x, y, width, height, margin, tooltip, pressedCallback, highlightCallback)
	local buttonFrame = {}
	
	buttonFrame.parentFrame = parentFrame
	buttonFrame.name = name
	buttonFrame.tooltip = tooltip
	buttonFrame.margin = margin
	buttonFrame.hasTexture = false
	buttonFrame.frame = UI.CreateFrame("Texture", "XMacros.Button", parentFrame)
	buttonFrame.frame:SetSecureMode("restricted")
	buttonFrame.frame:SetVisible(false)
	buttonFrame.frame:SetTexture(XMacros.name, "Textures/ButtonBorder.png")
	buttonFrame.frame:SetWidth(width)
	buttonFrame.frame:SetHeight(height)
	buttonFrame.pressedCallback = pressedCallback
	buttonFrame.highlightCallback = highlightCallback
	buttonFrame.width = width
	buttonFrame.height = height
	buttonFrame.macros = {}
	buttonFrame.tooltips = {}
	buttonFrame.frame:SetMouseMasking("limited")
	XMacros.Button.SetPosition(buttonFrame, x, y)

	buttonFrame.textureFrame = UI.CreateFrame("Texture", "XMacrosButtonTexture", buttonFrame.frame)
	buttonFrame.textureFrame:SetSecureMode("normal")
	buttonFrame.textureFrame:SetPoint("CENTER", buttonFrame.frame, "CENTER")
	buttonFrame.textureFrame:SetTexture(XMacros.name, "Textures/Click.png")
	buttonFrame.textureFrame:SetWidth(buttonFrame.width - 2 * buttonFrame.margin)
	buttonFrame.textureFrame:SetHeight(buttonFrame.height - 2 * buttonFrame.margin)
	buttonFrame.textureFrame:SetAlpha(0.3)

	buttonFrame.mouseoverFrame = UI.CreateFrame("Texture", "XMacrosMouseover", buttonFrame.frame)
	buttonFrame.mouseoverFrame:SetSecureMode("normal")
	buttonFrame.mouseoverFrame:SetVisible(false)
	buttonFrame.mouseoverFrame:SetLayer(buttonFrame.textureFrame:GetLayer() + 1)
	buttonFrame.mouseoverFrame:SetPoint("CENTER", buttonFrame.frame, "CENTER")
	buttonFrame.mouseoverFrame:SetTexture(XMacros.name, "Textures/ButtonMouseOver.png")
	buttonFrame.mouseoverFrame:SetWidth(width - 2 * buttonFrame.margin)
	buttonFrame.mouseoverFrame:SetHeight(height - 2 * buttonFrame.margin)
	
	return buttonFrame
end

function XMacros.Button.GetFrame(buttonFrame)
	return buttonFrame.frame
end

function XMacros.Button.GetName(buttonFrame)
	return buttonFrame.name
end

function XMacros.Button.SetName(buttonFrame, name)
	buttonFrame.name = name
end

function XMacros.Button.SetPosition(buttonFrame, x, y)
	buttonFrame.x = x
	buttonFrame.y = y
	buttonFrame.frame:SetPoint("TOPLEFT", buttonFrame.parentFrame, "TOPLEFT", x, y)
end

function XMacros.Button.SetAbilityName(buttonFrame, abilityName)
	if not buttonFrame.pressedCallback then
		XMacros.Button.ClearAbility(buttonFrame)

		if not buttonFrame.countdownFrame then
			local fontSize = XMacros.Utils.Round(buttonFrame.mouseoverFrame:GetWidth() / 16, 0) * 6
			local layer = buttonFrame.textureFrame:GetLayer() + 1

			buttonFrame.countShadowFrame = UI.CreateFrame("Text", "countShadowFrame", buttonFrame.frame)
			buttonFrame.countShadowFrame:SetSecureMode("normal")
			buttonFrame.countShadowFrame:SetVisible(false)
			buttonFrame.countShadowFrame:SetLayer(layer)
			buttonFrame.countShadowFrame:SetPoint("BOTTOMRIGHT", buttonFrame.frame, "BOTTOMRIGHT", -0, -2)
			buttonFrame.countShadowFrame:SetFontSize(fontSize)
			buttonFrame.countShadowFrame:SetFontColor(0,0,0,0.5)
			
			buttonFrame.countdownFrame = UI.CreateFrame("Text", "countdownFrame", buttonFrame.frame)
			buttonFrame.countdownFrame:SetSecureMode("normal")
			buttonFrame.countdownFrame:SetVisible(false)
			buttonFrame.countdownFrame:SetLayer(layer + 1)
			buttonFrame.countdownFrame:SetPoint("BOTTOMRIGHT", buttonFrame.frame, "BOTTOMRIGHT", -2, 0)
			buttonFrame.countdownFrame:SetFontSize(fontSize)
		end
	end
	
	buttonFrame.abilityName = abilityName
	if abilityName and not buttonFrame.pressedCallback then
		local cooldown = XMacros.AbilityManager.GetCooldown(abilityName)
		if cooldown and cooldown > 0 then
				XMacros.Button.AbilityStarted(buttonFrame)
		end
		
		buttonFrame.abilityStartID = XMacros.AbilityManager.AddAbilityBeginCallback(abilityName, function(a, inDuration, frameTime) XMacros.Button.AbilityStarted(buttonFrame, inDuration, frameTime) end)
	end
end

function XMacros.Button.ClearAbility(buttonFrame)
	if buttonFrame.abilityStartID then
		XMacros.AbilityManager.RemoveAbilityCallback(buttonFrame.abilityStartID)
		buttonFrame.abilityStartID = nil
	end
	if buttonFrame.abilityEndID then
		XMacros.AbilityManager.RemoveAbilityCallback(buttonFrame.abilityEndID)
		buttonFrame.abilityEndID = nil
	end
	
	if buttonFrame.countdownFrame then
		buttonFrame.countShadowFrame:SetVisible(false)
		buttonFrame.countdownFrame:SetVisible(false)
	end
	
	XMacros.Button.ClearAbilityTimer(buttonFrame)
end

function XMacros.Button.ClearAbilityTimer(buttonFrame)
	if buttonFrame.abilityTimer then
		XMacros.Timer.RemoveCallback(buttonFrame.abilityTimer)
		buttonFrame.abilityTimer = nil
	end
end

function XMacros.Button.GetAbilityName(buttonFrame)
	return buttonFrame.abilityName
end

function XMacros.Button.AbilityStarted(buttonFrame, inDuration)
	XMacros.Button.ClearAbilityTimer(buttonFrame)
	
	if not inDuration then
		return
	end
	
	buttonFrame.currentDuration = inDuration
	XMacros.Button.StartAbilityTimer(buttonFrame, currentTime)
	XMacros.Button.ShowAbility(buttonFrame, currentTime)
end

function XMacros.Button.AbilityFinished(buttonFrame)
	XMacros.Button.ClearAbilityTimer(buttonFrame)
	buttonFrame.countShadowFrame:SetVisible(false)
	buttonFrame.countdownFrame:SetVisible(false)
	buttonFrame.currentDuration = nil
end

function XMacros.Button.StartAbilityTimer(buttonFrame)
	if buttonFrame.currentDuration and buttonFrame.currentDuration > 0 then
		local remainingDuration = XMacros.Utils.Round(buttonFrame.currentDuration, 1)
		local timerDuration = 0
		if remainingDuration >= 3600 then
			local hours = math.floor(remainingDuration / 3600)
			if hours > 1 then
				timerDuration = 3600
			else
				timerDuration = remainingDuration - 60
			end
		elseif remainingDuration >= 60 then
			local mins = math.floor(remainingDuration / 60)
			if mins > 2 then
				timerDuration = 60
			elseif mins == 2 then
				timerDuration = 61
			else
				timerDuration = 1
			end
		elseif remainingDuration >= 0 then
			local unit, dec = math.modf(remainingDuration)
			if dec > 0.1 then
				timerDuration = dec
			else
				timerDuration = 1
			end
		else
			return
		end

		if timerDuration < 0.1 then
			print("Reset timer=" .. timerDuration .. " cd=" .. cooldown .. " rem=" .. remainingDuration)
			timerDuration = 0.1
		end
		
		XMacros.Timer.AddCallback(buttonFrame.abilityName .. "Timer", timerDuration, function(frameTime) XMacros.Button.AbilityStarted(buttonFrame, remainingDuration - timerDuration, frameTime) end, false)
	end
end

function XMacros.Button.ShowAbility(buttonFrame)
	local cdVisible = false
	local remainingDuration = buttonFrame.currentDuration
	if remainingDuration and remainingDuration > 0 then
		cdVisible = true
		local displayString = ""
		if remainingDuration >= 3600 then
			displayString = tostring(math.floor(remainingDuration / 3600)) .. "h"
		elseif remainingDuration >= 60 then
			displayString = tostring(math.floor(remainingDuration / 60)) .. "m"
		elseif remainingDuration >= 0 then
			displayString = tostring(remainingDuration) .. "s"
		else
			cdVisible = false
		end
		
		if cdVisible == true then
			if remainingDuration <= 3 then
				buttonFrame.countdownFrame:SetFontColor(1, 0, 0)
			else
				buttonFrame.countdownFrame:SetFontColor(1, 1, 0.6)
			end
		end
			
		buttonFrame.countShadowFrame:SetText(displayString)
		buttonFrame.countdownFrame:SetText(displayString)
	end
	buttonFrame.countShadowFrame:SetVisible(cdVisible)
	buttonFrame.countdownFrame:SetVisible(cdVisible)
	
	if cdVisible == false then
		XMacros.Button.AbilityFinished(buttonFrame)
	end
end

function XMacros.Button.SetTexture(buttonFrame, textureAddon, textureName)
	buttonFrame.textureAddon = textureAddon
	buttonFrame.textureIcon = textureName
	buttonFrame.textureFrame:SetTexture(textureAddon, textureName)
	buttonFrame.hasTexture = true
end

function XMacros.Button.SetAlpha(buttonFrame, alpha)
	buttonFrame.frame:SetAlpha(alpha)
end

function XMacros.Button.GetTexture(buttonFrame)
	if buttonFrame.hasTexture == true then
		local addon, icon = buttonFrame.textureFrame:GetTexture()
		return addon, icon
	else
		return "", ""
	end
end

function XMacros.Button.HasTexture(buttonFrame)
	return buttonFrame.hasTexture == true
end

function XMacros.Button.AddMacro(buttonFrame, buttonName, category, macroName, macro)
	if category then
		buttonFrame.macros[buttonName] = { category = category, name = macroName }
	end
	
	if macro then
		buttonFrame.frame.Event[buttonName .. "Click"] = macro
		buttonFrame.frame.Event.MouseIn = function () XMacros.Button.SetHighlightVisible(buttonFrame, true) end
		buttonFrame.frame.Event.MouseOut = function () XMacros.Button.MouseUp(buttonFrame, false) XMacros.Button.SetHighlightVisible(buttonFrame, false) end
	else
		buttonFrame.frame.Event[buttonName .. "Upoutside"] = function () XMacros.Button.MouseUp(buttonFrame) end
	end
	
	buttonFrame.frame.Event[buttonName .. "Up"] = function () XMacros.Button.MouseUp(buttonFrame, buttonName) end
	buttonFrame.frame.Event[buttonName .. "Down"] = function () XMacros.Button.MouseDown(buttonFrame) end
	
	buttonFrame.textureFrame:SetAlpha(1)
end

function XMacros.Button.GetMacro(buttonFrame, buttonName)
	local macro = buttonFrame.macros[buttonName]
	if macro then
		return macro.category, macro.name
	end
end

function XMacros.Button.MouseDown(buttonFrame)
	local shrinkX = math.floor(buttonFrame.width / 8)
	if shrinkX < 1 then shrinkX = 1 end
	local shrinkY = math.floor(buttonFrame.height / 8)
	if shrinkY < 1 then shrinkY = 1 end
	buttonFrame.textureFrame:SetWidth(buttonFrame.width - 2 * buttonFrame.margin - shrinkX)
	buttonFrame.textureFrame:SetHeight(buttonFrame.height - 2 * buttonFrame.margin - shrinkY)
end

function XMacros.Button.MouseUp(buttonFrame, buttonName)
	buttonFrame.textureFrame:SetWidth(buttonFrame.width - 2 * buttonFrame.margin)
	buttonFrame.textureFrame:SetHeight(buttonFrame.height - 2 * buttonFrame.margin)
	
	if buttonName and buttonFrame.pressedCallback then
		buttonFrame.pressedCallback(buttonFrame, buttonName)
	end
end

function XMacros.Button.SetHighlightVisible(buttonFrame, visible)
	if buttonFrame.pressedCallback then
		buttonFrame.mouseoverFrame:SetVisible(false)
		return
	end
	
	local oldVisible = buttonFrame.mouseoverFrame:GetVisible()
	buttonFrame.mouseoverFrame:SetVisible(visible)
	if buttonFrame.tooltip then
		if visible == true then
			local abilityList = nil
			local messages = {}
			for i = 1, #XMacros.Button.buttonNames do
				local macro = buttonFrame.macros[XMacros.Button.buttonNames[i]]
				if macro then
					local inserted = false
					if macro.category == "Ability" then
						if not buttonFrame.tooltips[macro.name] then
							if not abilityList then
								abilityList = XMacros.AbilityManager.GetAbilities()
							end
							
							if abilityList then
								for _, ability in pairs(abilityList) do
									if ability.name == macro.name then
										if ability.description then
											buttonFrame.tooltips[macro.name] = ability.description
										end
										
										break
									end
								end
							end
						end
					end

					if buttonFrame.tooltips[macro.name] then
						inserted = true
						table.insert(messages, XMacros.Button.buttonAbbrev[i] .. ": " .. buttonFrame.tooltips[macro.name])
					end
					
					if inserted == false then
						table.insert(messages, XMacros.Button.buttonAbbrev[i] .. ": " .. macro.category .. "/" .. macro.name)
					end
				end
			end
			
			if #messages > 0 then
				XMacros.Tooltip.ShowTooltip(buttonFrame.tooltip, buttonFrame.x, buttonFrame.y, messages)
			end
		else
			XMacros.Tooltip.HideTooltip(buttonFrame.tooltip, buttonFrame.x, buttonFrame.y)
		end
	end
	
	if buttonFrame.highlightCallback and oldVisible ~= visible then
		if visible == true then
			buttonFrame.highlightCallback(1)
		else
			buttonFrame.highlightCallback(-1)
		end
	end
end

function XMacros.Button.SetVisible(buttonFrame, visible)
	if visible == false then
		XMacros.Button.SetHighlightVisible(buttonFrame, false)
	else
		local mouse = Inspect.Mouse()
		local mouseDown = false
		if mouse.x and mouse.y then
			local x = buttonFrame.frame:GetLeft()
			local y = buttonFrame.frame:GetTop()
			if mouse.x >= x and mouse.x <= x + buttonFrame.width and
			   mouse.y >= y and mouse.y <= y + buttonFrame.height then
				XMacros.Button.SetHighlightVisible(buttonFrame, true)
				
				if mouse.left == true then
					mouseDown = true
				end
			else
				XMacros.Button.SetHighlightVisible(buttonFrame, false)
			end
		end
		
		if mouseDown == true then
			XMacros.Button.MouseDown(buttonFrame)
		else
			XMacros.Button.MouseUp(buttonFrame, false)
		end
	end
	buttonFrame.frame:SetVisible(visible)
end

