-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.ButtonEditor = {}
XMacros.ButtonEditor.name = "XMacros.ButtonEditor"
XMacros.ButtonEditor.buttonNames = { "Left","Middle", "Right", "Mouse4","Mouse5" }

--
-- Main functions
--

--
-- UI functions
--

-- ability |name| /find icon/ /browse icons/
-- Left macro |text| /Choose/
-- Middle macro |text| /Choose/
-- Right macro |text| /Choose/
-- Mouse4 macro |text| /Choose/
-- Mouse5 macro |text| /Choose/
--
-- Resize Bar icons
-- Delete Button Bar

function XMacros.ButtonEditor.Create(parentFrame)
	local buttonEditor = {}
	buttonEditor.parentFrame = parentFrame
	
	buttonEditor.width = 650
	buttonEditor.height = 420
	buttonEditor.x = math.ceil(UIParent:GetWidth() / 2 - buttonEditor.width)
	buttonEditor.y = math.ceil(UIParent:GetWidth() / 2 - buttonEditor.height / 2)
	buttonEditor.windowFrame = UI.CreateFrame("SimpleWindow", "ButtonEditor", parentFrame)
	buttonEditor.windowFrame:SetLayer(parentFrame:GetLayer() + 2)
	buttonEditor.windowFrame:SetVisible(false)
	buttonEditor.windowFrame:SetSecureMode("normal")
	buttonEditor.windowFrame:SetCloseButtonVisible(true)
	buttonEditor.windowFrame:SetTitle("Button Editor")
	buttonEditor.windowFrame:SetWidth(buttonEditor.width)
	buttonEditor.windowFrame:SetHeight(buttonEditor.height)
	buttonEditor.windowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", buttonEditor.x, buttonEditor.height)
	buttonEditor.windowFrame.Event.Close = function() XMacros.ButtonEditor.WindowClose(buttonEditor) end
	buttonEditor.x = buttonEditor.windowFrame:GetLeft()
	buttonEditor.y = buttonEditor.windowFrame:GetTop()
	
	local labelWidth = 90
	local abilityFrame = UI.CreateFrame("Text", "AbilityFrame", buttonEditor.windowFrame)
	abilityFrame:SetPoint("TOPLEFT", buttonEditor.windowFrame, "TOPLEFT", 40, 60)
	abilityFrame:SetText("Ability Name")
	abilityFrame:SetWidth(labelWidth)
	
	local textWidth = 150
	buttonEditor.abilityText = UI.CreateFrame("RiftTextfield", "AbilityText", buttonEditor.windowFrame)
	buttonEditor.abilityText:SetPoint("TOPLEFT", abilityFrame, "TOPRIGHT", 20, 0)
	buttonEditor.abilityText:SetText("")
	buttonEditor.abilityText:SetWidth(textWidth)
	buttonEditor.abilityText:SetBackgroundColor(0.15, 0.15, 0.15, 0.4)
	Library.LibSimpleWidgets.SetBorder(buttonEditor.abilityText, 1, 0.5, 0.5, 0.5, 1)

	buttonEditor.abilityTexture = UI.CreateFrame("Texture", "AbilityTexture", buttonEditor.windowFrame)
	buttonEditor.abilityTexture:SetPoint("TOPLEFT", buttonEditor.abilityText, "BOTTOMLEFT", 0, 10)

	buttonEditor.abilityButton = UI.CreateFrame("RiftButton", "AbilityButton", buttonEditor.windowFrame)
	buttonEditor.abilityButton:SetPoint("TOPLEFT", buttonEditor.abilityText, "TOPRIGHT", 20, -8)
	buttonEditor.abilityButton:SetText("Find Icon")
	buttonEditor.abilityButton.Event.LeftPress = function() XMacros.ButtonEditor.FindIcon(buttonEditor) end

	buttonEditor.iconButton = UI.CreateFrame("RiftButton", "IconButton", buttonEditor.windowFrame)
	buttonEditor.iconButton:SetPoint("TOPLEFT", buttonEditor.abilityButton, "TOPLEFT", 0, 30)
	buttonEditor.iconButton:SetText("Browse Icons")
	buttonEditor.iconButton.Event.LeftPress = function() XMacros.ButtonEditor.BrowseIcons(buttonEditor) end

	for i, buttonName in ipairs(XMacros.ButtonEditor.buttonNames) do
		local textFrame = UI.CreateFrame("Text", buttonName .. "LabelFrame", buttonEditor.windowFrame)
		textFrame:SetPoint("TOPLEFT", abilityFrame, "TOPLEFT", 0, 30 * (i + 1) + 20)
		textFrame:SetText(buttonName .. " Macro")
		textFrame:SetWidth(labelWidth)

		local frameName = buttonName .. "Macro"
		local thisFrame = UI.CreateFrame("Text", frameName, buttonEditor.windowFrame)
		thisFrame:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 20, 0)
		thisFrame:SetText("")
		thisFrame:SetWidth(textWidth)
		buttonEditor[frameName] = thisFrame
		local previousFrame = thisFrame

		local frameName = buttonName .. "ChooseButton"
		local thisFrame = UI.CreateFrame("RiftButton", frameName, buttonEditor.windowFrame)
		thisFrame:SetPoint("TOPLEFT", previousFrame, "TOPRIGHT", 20, -8)
		thisFrame:SetText("Choose")
		thisFrame.Event.LeftPress = function() XMacros.ButtonEditor.ChooseMacro(buttonEditor, buttonName) end
		buttonEditor[frameName] = thisFrame
		local previousFrame = thisFrame

		local frameName = buttonName .. "ClearButton"
		local thisFrame = UI.CreateFrame("RiftButton", frameName, buttonEditor.windowFrame)
		thisFrame:SetPoint("TOPLEFT", previousFrame, "TOPRIGHT", 20, 0)
		thisFrame:SetText("Clear")
		thisFrame.Event.LeftPress = function() XMacros.ButtonEditor.ClearMacro(buttonEditor, buttonName) end
		buttonEditor[frameName] = thisFrame
	end
	
	local textFrame = UI.CreateFrame("Text", "ResizeFrame", buttonEditor.windowFrame)
	textFrame:SetPoint("TOPLEFT", abilityFrame, "TOPLEFT", 0, 30 * (#XMacros.ButtonEditor.buttonNames + 2) + 20)
	textFrame:SetText("Resize Bar Icons")
	textFrame:SetWidth(labelWidth)
	
	buttonEditor.iconSizeSlider = UI.CreateFrame("SimpleSlider", "IconSize", buttonEditor.windowFrame)
	buttonEditor.iconSizeSlider:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 20, 0)
	buttonEditor.iconSizeSlider:SetRange(16, 256)
	buttonEditor.iconSizeSlider:SetPosition(52)
	buttonEditor.iconSizeSlider:SetWidth(300)
	
	buttonEditor.okButton = UI.CreateFrame("RiftButton", "OKButton", buttonEditor.windowFrame)
	buttonEditor.okButton:SetPoint("BOTTOMLEFT", buttonEditor.windowFrame, "BOTTOMLEFT", 20, -20)
	buttonEditor.okButton:SetText("OK")
	buttonEditor.okButton.Event.LeftPress = function() XMacros.ButtonEditor.WindowClose(buttonEditor, true) end	

	buttonEditor.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", buttonEditor.windowFrame)
	buttonEditor.cancelButton:SetPoint("BOTTOMRIGHT", buttonEditor.windowFrame, "BOTTOMRIGHT", -20, -20)
	buttonEditor.cancelButton:SetText("Cancel")
	buttonEditor.cancelButton.Event.LeftPress = function() XMacros.ButtonEditor.WindowClose(buttonEditor) end	

	buttonEditor.deleteButton = UI.CreateFrame("RiftButton", "DeleteButton", buttonEditor.windowFrame)
	buttonEditor.deleteButton:SetText("Delete Bar")
	local position = math.floor(buttonEditor.width / 2 - buttonEditor.deleteButton:GetWidth() / 2)
	buttonEditor.deleteButton:SetPoint("BOTTOMLEFT", buttonEditor.windowFrame, "BOTTOMLEFT", position, -20)
	buttonEditor.deleteButton.Event.LeftPress = function() XMacros.ButtonEditor.DeleteBar(buttonEditor) end	

	buttonEditor.macroPicker = XMacros.MacroPicker.Create(buttonEditor.parentFrame, buttonEditor.windowFrame)
	buttonEditor.txtPicker = XMacros.TexturePicker.Create(buttonEditor.parentFrame, buttonEditor.windowFrame, function(a, name, addon, path) XMacros.ButtonEditor.BrowseCallback(buttonEditor, name, addon, path) end)
	buttonEditor.wrnWindow = XMacros.WarningWindow.CreateWarningWindow(buttonEditor.parentFrame, buttonEditor.windowFrame)
	buttonEditor.cfmWindow = XMacros.ConfirmationWindow.CreateConfirmationWindow(buttonEditor.parentFrame, buttonEditor.windowFrame)
	
	return buttonEditor
end

function XMacros.ButtonEditor.WindowClose(buttonEditor, okPressed)
	if not buttonEditor then
		return
	end
	
	buttonEditor.abilityText:SetKeyFocus(false)
	XMacros.TexturePicker.Hide(buttonEditor.txtPicker)
	buttonEditor.windowFrame:SetVisible(false)
	if buttonEditor.callback then
		if okPressed == true then
			local results = {}
			local addon, icon = buttonEditor.abilityTexture:GetTexture()
			results.iconAddon = addon
			results.iconPath = icon
			
			results.macros = {}
			for i, buttonName in ipairs(XMacros.ButtonEditor.buttonNames) do
				local frameName = buttonName .. "Macro"
				local thisFrame = buttonEditor[frameName]
				local macroText = thisFrame:GetText()
				if macroText and string.len(macroText) > 0 then
					local parts = XMacros.Utils.Split(macroText, '/')
					if parts and #parts == 2 then
						results.macros[buttonName] = { category = parts[1], macro = parts[2] }
					end
				end
			end
			
			local iconSize = tonumber(buttonEditor.iconSizeSlider:GetPosition())
			if iconSize then
				results.iconSize = iconSize
			end
			
			local abilityString = XMacros.Utils.Trim(buttonEditor.abilityText:GetText())
			if string.len(abilityString) > 0 then
				local abilityName = XMacros.TexturePicker.FindAbilityName(abilityString)
				if abilityName then
					results.abilityName = abilityName
				else
					results.abilityName = abilityString
				end
			end
			
			buttonEditor.callback(buttonEditor, results)
		else
			buttonEditor.callback(buttonEditor)
		end
	end
end

function XMacros.ButtonEditor.Show(buttonEditor, buttonSets, buttonBar, row, col, fullCategoryNames, fullMacroNames, callback, refreshCallback)
	buttonEditor.buttonBar = buttonBar
	buttonEditor.buttonSets = buttonSets
	buttonEditor.row = row
	buttonEditor.col = col
	buttonEditor.callback = callback
	buttonEditor.refreshCallback = refreshCallback
	buttonEditor.buttonName = XMacros.ButtonBar.GetButtonName(row, col)
	buttonEditor.button = XMacros.ButtonBar.GetButton(buttonBar, row, col)
	
	local addon, icon = XMacros.Button.GetTexture(buttonEditor.button)
	if icon and string.len(icon) > 0 then
		buttonEditor.abilityTexture:SetTexture(addon, icon)
		buttonEditor.abilityTexture:SetVisible(true)
	else
		buttonEditor.abilityTexture:SetVisible(false)
	end
	
	XMacros.MacroPicker.SetMacros(buttonEditor.macroPicker, fullCategoryNames, fullMacroNames)
	
	for _, macroName in ipairs(XMacros.ButtonEditor.buttonNames) do
		local category, name = XMacros.Button.GetMacro(buttonEditor.button, macroName)
		
		local textString = ""
		if category and name then
			textString = category .. "/" .. name
		end
		local frameName = macroName .. "Macro"
		local thisFrame = buttonEditor[frameName]
		thisFrame:SetText(textString)
	end

	local abilityName = XMacros.Button.GetAbilityName(buttonEditor.button)
	if abilityName then
		buttonEditor.abilityText:SetText(abilityName)
	else
		buttonEditor.abilityText:SetText("")
	end
	
	buttonEditor.iconSizeSlider:SetPosition(tonumber(buttonBar.buttonSize))
	XMacros.ButtonEditor.SetEnabled(buttonEditor, true)
	buttonEditor.windowFrame:SetVisible(true)
end

function XMacros.ButtonEditor.SetEnabled(buttonEditor, enabled)
	buttonEditor.abilityButton:SetEnabled(enabled)
	buttonEditor.iconButton:SetEnabled(enabled)
	buttonEditor.okButton:SetEnabled(enabled)
	buttonEditor.deleteButton:SetEnabled(enabled)
	buttonEditor.cancelButton:SetEnabled(enabled)

	for _, buttonName in ipairs(XMacros.ButtonEditor.buttonNames) do
		local frameName = buttonName .. "ChooseButton"
		local thisFrame = buttonEditor[frameName]
		thisFrame:SetEnabled(enabled)

		local frameName = buttonName .. "ClearButton"
		local thisFrame = buttonEditor[frameName]
		thisFrame:SetEnabled(enabled)
	end
end

function XMacros.ButtonEditor.DeleteBar(buttonEditor)
	XMacros.ButtonSets.DeleteBar(buttonEditor.buttonSets, buttonEditor.buttonBar.setName, buttonEditor.buttonBar.barName)
	if buttonEditor.refreshCallback then
		buttonEditor.refreshCallback()
	end
	XMacros.ButtonEditor.WindowClose(buttonEditor)
end

function XMacros.ButtonEditor.AbilityChosen(buttonEditor, macroString, buttonFrame, okPressed)
	XMacros.ButtonEditor.SetEnabled(buttonEditor, true)
	if okPressed == true then
		buttonFrame:SetText(macroString)
	end
end

local function SetAbility(buttonEditor, abilityName)
	local abilityString = XMacros.TexturePicker.FindAbilityName(abilityName)
	if abilityString then
		buttonEditor.abilityText:SetText(abilityString)
		
		local macroString = XMacros.MacroManager.AbilityCategory .. "/" .. abilityString
		local frameName = "LeftMacro"
		local buttonFrame = buttonEditor[frameName]
		local macroText = buttonFrame:GetText()
		if macroText and string.len(macroText) > 0 then
			XMacros.ButtonEditor.SetEnabled(buttonEditor, false)
			XMacros.ConfirmationWindow.ShowConfirmationWindow(buttonEditor.cfmWindow, "Do you want to set the Left mouse button\nto be " .. abilityString .. "?", function(a, okPressed) XMacros.ButtonEditor.AbilityChosen(buttonEditor, macroString, buttonFrame, okPressed) end)
		else
			buttonFrame:SetText(macroString)
		end
	end
end

function XMacros.ButtonEditor.FindIcon(buttonEditor)
	local abilityName = buttonEditor.abilityText:GetText()
	if abilityName then
		local iconPath = XMacros.TexturePicker.FindIconForAbility(abilityName)
		if iconPath then
			buttonEditor.iconPath = iconPath
			buttonEditor.abilityTexture:SetTexture("Rift", iconPath)
			buttonEditor.abilityTexture:SetVisible(true)
			
			SetAbility(buttonEditor, abilityName)
		else
			local textureArray = XMacros.TexturePicker.FilterIcons(abilityName)
			if textureArray == nil or #textureArray < 1 then
				textureArray = XMacros.TexturePicker.GetTextureArray()
			end
			XMacros.ButtonEditor.SetEnabled(buttonEditor, false)
			XMacros.TexturePicker.Show(buttonEditor.txtPicker, textureArray, XMacros.ButtonEditor.RemoveIconCallback)
		end
	end
end

function XMacros.ButtonEditor.BrowseCallback(buttonEditor, name, addon, path)
	XMacros.ButtonEditor.SetEnabled(buttonEditor, true)
	if name and path then
		buttonEditor.iconPath = path
		buttonEditor.abilityTexture:SetTexture(addon, path)
		buttonEditor.abilityTexture:SetVisible(true)
		buttonEditor.abilityText:SetText("")
		
		local nameParts = XMacros.Utils.Split(name, "/")
		if #nameParts == 2 then
			SetAbility(buttonEditor, nameParts[2])
		end
	end
end

function XMacros.ButtonEditor.BrowseIcons(buttonEditor)
	local textureArray = XMacros.TexturePicker.GetTextureArray()
	XMacros.ButtonEditor.SetEnabled(buttonEditor, false)
	XMacros.TexturePicker.Show(buttonEditor.txtPicker, textureArray, XMacros.ButtonEditor.RemoveIconCallback)
end

function XMacros.ButtonEditor.RemoveIconCallback(name)
	if name then
		XMacros.RemoveAdditionalIcon(name)
	end
end

function XMacros.ButtonEditor.MacroChosen(buttonEditor, buttonName, macroString)
	XMacros.ButtonEditor.SetEnabled(buttonEditor, true)
	if macroString then
		local frameName = buttonName .. "Macro"
		local thisFrame = buttonEditor[frameName]
		thisFrame:SetText(macroString)
	end
end

function XMacros.ButtonEditor.ChooseMacro(buttonEditor, buttonName)
	XMacros.ButtonEditor.SetEnabled(buttonEditor, false)
	XMacros.MacroPicker.Show(buttonEditor.macroPicker, function(a, macroString) XMacros.ButtonEditor.MacroChosen(buttonEditor, buttonName, macroString) end)
end

function XMacros.ButtonEditor.ClearMacro(buttonEditor, buttonName)
	local frameName = buttonName .. "Macro"
	local thisFrame = buttonEditor[frameName]
	thisFrame:SetText("")
end
