-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.EntryWindow = {}
XMacros.EntryWindow.name = "XMacros.EntryWindow"

--
-- Main functions
--

--
-- UI functions
--

local function DrawBorder(entWindow, isValid)
	if isValid == true then
		Library.LibSimpleWidgets.SetBorder(entWindow.textField, 1, 0.5, 0.5, 0.5, 1)
	else
		Library.LibSimpleWidgets.SetBorder(entWindow.textField, 1, 1, 0, 0, 1)
	end
end

local function CheckText(entWindow)
	local isValid = true
	local text = entWindow.textField:GetText()
	if not text or text == "" then
		isValid = false
	elseif entWindow.regex and string.find(text, entWindow.regex) ~= nil then
		isValid = false
	end

	entWindow.entryWindowButton:SetEnabled(isValid)
	DrawBorder(entWindow, isValid)
end

function XMacros.EntryWindow.CreateEntryWindow(parentFrame, positonFrame)
	local entWindow = {}
	entWindow.entryWindow = UI.CreateFrame("RiftWindow", "entWindow", parentFrame)
	entWindow.entryWindow:SetVisible(false)
	entWindow.entryWindow:SetTitle("Entry window")
	entWindow.entryWindow:SetHeight(350)
	entWindow.entryWindow:SetPoint("CENTER", positonFrame, "CENTER")
	entWindow.entryWindow:SetLayer(positonFrame:GetLayer() + 1)
	
	entWindow.entryWindowButton = UI.CreateFrame("RiftButton", "EntryYesButton", entWindow.entryWindow)
	entWindow.entryWindowButton:SetText("OK")
	entWindow.entryWindowButton:SetPoint("BOTTOMLEFT", entWindow.entryWindow, "BOTTOMLEFT", 20, -20)
	entWindow.entryWindowButton.Event.LeftPress = function() XMacros.EntryWindow.EntryWindowClose(entWindow, true) end
	
	entWindow.entryWindowCancelButton = UI.CreateFrame("RiftButton", "EntryYesButton", entWindow.entryWindow)
	entWindow.entryWindowCancelButton:SetText("Cancel")
	entWindow.entryWindowCancelButton:SetPoint("BOTTOMRIGHT", entWindow.entryWindow, "BOTTOMRIGHT", -20, -20)
	entWindow.entryWindowCancelButton.Event.LeftPress = function() XMacros.EntryWindow.EntryWindowClose(entWindow, false) end
	
	entWindow.entryLabel = UI.CreateFrame("Text", "EntryLabel", entWindow.entryWindow)
	entWindow.entryLabel:SetPoint("TOPLEFT", entWindow.entryWindow, "TOPLEFT", 20, 60)
	entWindow.entryLabel:SetPoint("TOPRIGHT", entWindow.entryWindow, "TOPRIGHT", -20, 60)
	entWindow.entryLabel:SetWordwrap(true)
	entWindow.entryLabel:SetText("Dummy message")
	
	entWindow.textField = UI.CreateFrame("RiftTextfield", "EntryText", entWindow.entryWindow)
	entWindow.textField:SetPoint("TOPLEFT", entWindow.entryLabel, "BOTTOMLEFT", 0, 5)
	entWindow.textField.Event.TextfieldChange = function() CheckText(entWindow) end
	
	entWindow.entryWindow:SetWidth(entWindow.entryWindowButton:GetWidth() * 2 + 60)

	entWindow.entryCallback = nil
	return entWindow
end

function XMacros.EntryWindow.ShowEntryWindow(entWindow, message, textValue, callback, regex)
	if message ~= nil then
		entWindow.entryLabel:SetText(message)
	else
		entWindow.entryLabel:SetText("")
	end
	
	if textValue ~= nil then
		entWindow.textField:SetText(textValue)
	else
		entWindow.textField:SetText("")
	end
	
	entWindow.regex = regex
	entWindow.textField:SetKeyFocus(true)
	entWindow.entryCallback = callback
	CheckText(entWindow)
	entWindow.entryWindow:SetVisible(true)
end

function XMacros.EntryWindow.EntryWindowClose(entWindow, okPressed)
	entWindow.textField:SetKeyFocus(false)
	entWindow.entryWindow:SetVisible(false)
	if entWindow.entryCallback then
		if okPressed == true then
			entWindow.entryCallback(entWindow, entWindow.textField:GetText())
		else
			entWindow.entryCallback(entWindow)
		end
	end
end
