-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.MacroEditor = {}
XMacros.MacroEditor.name = "XMacros.MacroEditor"

--
-- Main functions
--

--
-- UI functions
--
local function DrawBorder(macroEditor, isValid)
	if isValid == true then
		Library.LibSimpleWidgets.SetBorder(macroEditor.nameText, 1, 0.5, 0.5, 0.5, 1)
	else
		Library.LibSimpleWidgets.SetBorder(macroEditor.nameText, 1, 1, 0, 0, 1)
	end
end

local function CheckText(macroEditor)
	local isValid = true
	local text = macroEditor.nameText:GetText()
	if not text or text == "" then
		isValid = false
	elseif string.find(text, "[^%a%d]+") ~= nil then
		isValid = false
	end

	macroEditor.saveButton:SetEnabled(isValid)
	DrawBorder(macroEditor, isValid)
end

function XMacros.MacroEditor.Create(parentFrame, callback, x, y, movedCallback)
	local macroEditor = {}
	macroEditor.parentFrame = parentFrame
	macroEditor.callback = callback
	macroEditor.movedCallback = movedCallback
	macroEditor.category = XMacros.MacroManager.DefaultCategory
	
	macroEditor.width = 835
	macroEditor.height = 620
	macroEditor.windowFrame = UI.CreateFrame("SimpleWindow", "MacroEditor", parentFrame)
	macroEditor.windowFrame:SetVisible(false)
	macroEditor.windowFrame:SetSecureMode("normal")
	macroEditor.windowFrame:SetCloseButtonVisible(true)
	macroEditor.windowFrame:SetTitle("Macro Editor")
	macroEditor.windowFrame:SetWidth(macroEditor.width)
	macroEditor.windowFrame:SetHeight(macroEditor.height)
	macroEditor.windowFrame.Event.Close = function() XMacros.MacroEditor.WindowClose(macroEditor, true) end
	
	if x >= 0 then
		macroEditor.x = x
		macroEditor.y = y
		macroEditor.windowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
	else
		macroEditor.windowFrame:SetPoint("CENTER", UIParent, "CENTER")
		macroEditor.x = macroEditor.windowFrame:GetLeft()
		macroEditor.y = macroEditor.windowFrame:GetTop()
	end
	
	local xMargin = 15
	macroEditor.selectBorder = UI.CreateFrame("SimpleBorderExt", macroEditor.windowFrame:GetName() .. "SelectBorder", macroEditor.windowFrame)
	macroEditor.selectBorder:SetLayer(3)
	macroEditor.selectBorder:SetPoint("TOPLEFT", macroEditor.windowFrame, "TOPLEFT", xMargin, 50)
	macroEditor.selectBorder:SetPoint("BOTTOMRIGHT", macroEditor.windowFrame, "BOTTOMLEFT", xMargin + 320, -20)
	macroEditor.selectContent = macroEditor.selectBorder:GetContent()
	
	macroEditor.setBorder = UI.CreateFrame("SimpleBorderExt", macroEditor.windowFrame:GetName() .. "EditBorder", macroEditor.windowFrame)
	macroEditor.setBorder:SetLayer(2)
	macroEditor.setBorder:SetPoint("TOPLEFT", macroEditor.selectBorder, "TOPRIGHT", 5, 0)
	macroEditor.setBorder:SetPoint("BOTTOMRIGHT", macroEditor.windowFrame, "BOTTOMRIGHT", -xMargin, -20)
	macroEditor.setContent = macroEditor.setBorder:GetContent()

	local textFrame = UI.CreateFrame("Text", "GridFrame", macroEditor.selectContent)
	textFrame:SetPoint("TOPLEFT", macroEditor.selectContent, "TOPLEFT", 5, xMargin)
	textFrame:SetText("Category")
	
	macroEditor.categoryButton = UI.CreateFrame("RiftButton", "CategoryButton", macroEditor.selectContent)
	macroEditor.categoryButton:SetPoint("TOPRIGHT", macroEditor.selectContent, "TOPRIGHT", -5, xMargin -8)
	macroEditor.categoryButton:SetText("Change")
	macroEditor.categoryButton.Event.LeftPress = function() XMacros.MacroEditor.ShowChooseCategory(macroEditor) end

	macroEditor.categoryText = UI.CreateFrame("Text", "CategoryText", macroEditor.selectContent)
	macroEditor.categoryText:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 10, 0)
	macroEditor.categoryText:SetPoint("TOPRIGHT", macroEditor.categoryButton, "TOPLEFT", -10, 8)
	macroEditor.categoryText:SetText("")
	Library.LibSimpleWidgets.SetBorder(macroEditor.categoryText, 1, 0.5, 0.5, 0.5, 1)

	macroEditor.prevButton = UI.CreateFrame("RiftButton", "PrevButton", macroEditor.selectContent)
	macroEditor.prevButton:SetPoint("BOTTOMLEFT", macroEditor.selectContent, "BOTTOMLEFT", 5, -xMargin + 8)
	macroEditor.prevButton:SetText("<<")
	macroEditor.prevButton.Event.LeftPress = function() XMacros.MacroEditor.PageTurn(macroEditor, -1) end

	macroEditor.nextButton = UI.CreateFrame("RiftButton", "NextButton", macroEditor.selectContent)
	macroEditor.nextButton:SetPoint("BOTTOMRIGHT", macroEditor.selectContent, "BOTTOMRIGHT", -5, -xMargin + 8)
	macroEditor.nextButton:SetText(">>")
	macroEditor.nextButton.Event.LeftPress = function() XMacros.MacroEditor.PageTurn(macroEditor, 1) end	

	macroEditor.deleteButton = UI.CreateFrame("RiftButton", "DeleteButton", macroEditor.setContent)
	macroEditor.deleteButton:SetPoint("BOTTOMRIGHT", macroEditor.setContent, "BOTTOMRIGHT", -5, -xMargin + 8)
	macroEditor.deleteButton:SetText("Delete")
	macroEditor.deleteButton.Event.LeftPress = function() XMacros.MacroEditor.DeleteMacro(macroEditor) end	

	macroEditor.undeleteButton = UI.CreateFrame("RiftButton", "UndeleteButton", macroEditor.setContent)
	macroEditor.undeleteButton:SetPoint("BOTTOMRIGHT", macroEditor.deleteButton, "BOTTOMLEFT", -10, 0)
	macroEditor.undeleteButton:SetText("Undo Delete")
	macroEditor.undeleteButton:SetVisible(true)
	macroEditor.undeleteButton.Event.LeftPress = function() XMacros.MacroEditor.UndeleteMacro(macroEditor) end

	local margin = 4
	local height = 20
	local rows = 18
	local width = macroEditor.nextButton:GetWidth() * 2 + 15
	macroEditor.gridFrame = UI.CreateFrame("Frame", "GridFrame", macroEditor.selectContent)
	macroEditor.gridFrame:SetPoint("TOPLEFT", textFrame, "BOTTOMLEFT", 0, 20)
	macroEditor.rowLayout = XMacros.RowLayout.Create(macroEditor.gridFrame, rows, 1, height, width, margin, margin)
	for row = 1, rows do
		local txtFrame = XMacros.RowLayout.CreateFrame(macroEditor.rowLayout, row, 1, "Text")
		if row % 2 == 1 then
			txtFrame:SetBackgroundColor(0, 0, 0.15)
		else
			txtFrame:SetBackgroundColor(0.15, 0.15, 0.15)
		end
	end

	XMacros.RowLayout.Redraw(macroEditor.rowLayout)
	Library.LibSimpleWidgets.SetBorder(macroEditor.gridFrame, 1, 0.5, 0.5, 0.5, 1)

	local textFrame = UI.CreateFrame("Text", "GridFrame", macroEditor.setContent)
	textFrame:SetPoint("TOPLEFT", macroEditor.setContent, "TOPLEFT", 5, xMargin)
	textFrame:SetText("Name")
	
	macroEditor.nameText = UI.CreateFrame("RiftTextfield", "CategoryText", macroEditor.setContent)
	macroEditor.nameText:SetPoint("TOPLEFT", textFrame, "TOPRIGHT", 10, 0)
	macroEditor.nameText:SetPoint("TOPRIGHT", macroEditor.setContent, "TOPRIGHT", -5, xMargin)
	macroEditor.nameText:SetBackgroundColor(0.15, 0.15, 0.15, 0.4)
	macroEditor.nameText:SetText("")
	macroEditor.nameText.Event.TextfieldChange = function() CheckText(macroEditor) end
	Library.LibSimpleWidgets.SetBorder(macroEditor.nameText, 1, 0.5, 0.5, 0.5, 1)

	macroEditor.editText = UI.CreateFrame("RiftTextfield", "EditField", macroEditor.setContent)
	macroEditor.editText:SetPoint("TOPLEFT", textFrame, "BOTTOMLEFT", 0, 20)
	macroEditor.editText:SetPoint("BOTTOMRIGHT", macroEditor.deleteButton, "TOPRIGHT", 0, -4)
	macroEditor.editText:SetBackgroundColor(0.15, 0.15, 0.15, 0.4)
	macroEditor.editText.Event.KeyType = function(a, key) XMacros.MacroEditor.TextFieldChanged(macroEditor, key) end
	Library.LibSimpleWidgets.SetBorder(macroEditor.editText, 1, 0.5, 0.5, 0.5, 1)

	macroEditor.saveButton = UI.CreateFrame("RiftButton", "SaveButton", macroEditor.setContent)
	macroEditor.saveButton:SetPoint("BOTTOMLEFT", macroEditor.setContent, "BOTTOMLEFT", 5, -xMargin + 8)
	macroEditor.saveButton:SetText("Save")
	macroEditor.saveButton.Event.LeftPress = function() XMacros.MacroEditor.SaveMacro(macroEditor) end
	
	local buttonWidth = math.floor((macroEditor.editText:GetWidth() - 20) / 3)
	macroEditor.saveButton:SetWidth(buttonWidth)
	macroEditor.undeleteButton:SetWidth(buttonWidth)
	macroEditor.deleteButton:SetWidth(buttonWidth)

	macroEditor.txtPicker = XMacros.TextPicker.Create(macroEditor.parentFrame, macroEditor.windowFrame, false)
	macroEditor.wrnWindow = XMacros.WarningWindow.CreateWarningWindow(macroEditor.parentFrame, macroEditor.windowFrame)
	macroEditor.cfmWindow = XMacros.ConfirmationWindow.CreateConfirmationWindow(macroEditor.parentFrame, macroEditor.windowFrame)
	
	return macroEditor
end

function XMacros.MacroEditor.WindowClose(macroEditor)
	macroEditor.undeleteButton:SetVisible(false)
	macroEditor.deletedMacroName = nil
	macroEditor.deletedMacro = nil

	macroEditor.nameText:SetKeyFocus(false)
	macroEditor.editText:SetKeyFocus(false)
	
	macroEditor.windowFrame:SetVisible(false)
	if macroEditor.callback then
		macroEditor.callback(macroEditor)
	end
end

function XMacros.MacroEditor.Show(macroEditor, manager)
	macroEditor.manager = manager
	
	macroEditor.undeleteButton:SetVisible(false)
	macroEditor.deletedMacroName = nil
	macroEditor.deletedMacro = nil
	
	macroEditor.categoryText:SetText(macroEditor.category)
	macroEditor.nameText:SetText("")
	macroEditor.editText:SetText("")
	
	macroEditor.macroName = nil
	macroEditor.page = 1

	XMacros.MacroEditor.UpdateMacroList(macroEditor)
	
	macroEditor.windowFrame:SetVisible(true)
end

function XMacros.MacroEditor.SetEnabled(macroEditor, enabled)
	macroEditor.categoryButton:SetEnabled(enabled)
	macroEditor.deleteButton:SetEnabled(enabled)
	macroEditor.undeleteButton:SetEnabled(enabled)
	macroEditor.saveButton:SetEnabled(enabled)
	
	if enabled == true then
		macroEditor.prevButton:SetEnabled(macroEditor.page ~= 1)
		macroEditor.nextButton:SetEnabled(macroEditor.page ~= macroEditor.maxPage)
	else
		macroEditor.prevButton:SetEnabled(enabled)
		macroEditor.nextButton:SetEnabled(enabled)
	end
end

function XMacros.MacroEditor.UpdateMacroList(macroEditor)
	local rows = XMacros.RowLayout.GetNumRows(macroEditor.rowLayout)
	macroEditor.macroNames = XMacros.MacroManager.GetMacroNames(macroEditor.manager, macroEditor.category)
	macroEditor.maxPage = math.ceil(#macroEditor.macroNames / rows)
	if macroEditor.maxPage == 0 then
		macroEditor.maxPage = 1
	end
	if macroEditor.page > macroEditor.maxPage then
		macroEditor.page = macroEditor.maxPage
	end

	XMacros.MacroEditor.RedrawMacroList(macroEditor)
end

function XMacros.MacroEditor.TextFieldChanged(macroEditor, key)
	if string.byte(key) == 13 then
		local newLines = 0
		local currentText = macroEditor.editText:GetText()
		for i = 1, string.len(currentText) do
			if string.byte(currentText, i) == 13 then
				newLines = newLines + 1
			end
		end
		
		if newLines < 34 then
			local cursor = macroEditor.editText:GetCursor()
			if cursor == 0 then
				macroEditor.editText:SetText("\n" .. macroEditor.editText:GetText())
			elseif cursor == string.len(macroEditor.editText:GetText()) then
				macroEditor.editText:SetText(macroEditor.editText:GetText() .. "\n")
			else
				local text = macroEditor.editText:GetText()
				macroEditor.editText:SetText(string.sub(text, 1, cursor) .. "\n" .. string.sub(text, cursor + 1))
			end
			macroEditor.editText:SetCursor(macroEditor.editText:GetCursor() + 1)
		end
	end
end

function XMacros.MacroEditor.ShowChooseCategory(macroEditor)
	XMacros.MacroEditor.SetEnabled(macroEditor, false)
	XMacros.TextPicker.Show(macroEditor.txtPicker, "Choose Macro Category", XMacros.MacroManager.GetCategoryNames(macroEditor.manager), function(a, category) XMacros.MacroEditor.ChooseCategory(macroEditor, category) end)
end

function XMacros.MacroEditor.ChooseCategory(macroEditor, category)
	XMacros.MacroEditor.SetEnabled(macroEditor, true)
	if category then
		macroEditor.category = category
		XMacros.MacroManager.GetCategory(macroEditor.manager, category)
		XMacros.MacroEditor.Show(macroEditor, macroEditor.manager)
	end
end

function XMacros.MacroEditor.DeleteMacro(macroEditor)
	local macroName = XMacros.Utils.Trim(macroEditor.nameText:GetText())
	local macro = XMacros.MacroManager.GetMacro(macroEditor.manager, macroEditor.category, macroName, true)
	if macro and string.len(macro) > 0 then
		macroEditor.deletedMacroName = macroName
		macroEditor.deletedMacro = macro
		XMacros.MacroManager.DeleteMacro(macroEditor.manager, macroEditor.category, macroName)
		
		macroEditor.undeleteButton:SetVisible(true)
		
		macroEditor.macroName = nil
		macroEditor.macro = nil
		macroEditor.nameText:SetText("")
		macroEditor.editText:SetText("")

		XMacros.MacroEditor.UpdateMacroList(macroEditor)
	end
end

function XMacros.MacroEditor.UndeleteMacro(macroEditor)
	if macroEditor.deletedMacroName then
		XMacros.MacroManager.AddMacro(macroEditor.manager, macroEditor.category, macroEditor.deletedMacroName, macroEditor.deletedMacro)
		
		macroEditor.undeleteButton:SetVisible(false)
		macroEditor.deletedMacroName = nil
		macroEditor.deletedMacro = nil

		XMacros.MacroEditor.UpdateMacroList(macroEditor)
	end
end

function XMacros.MacroEditor.SaveMacroCallback(macroEditor, macroName, macro, okPressed)
	if okPressed == true then
		XMacros.MacroManager.AddMacro(macroEditor.manager, macroEditor.category, macroName, macro)
		XMacros.MacroEditor.UpdateMacroList(macroEditor)
	end
	
	XMacros.MacroEditor.SetEnabled(macroEditor, true)
end

function XMacros.MacroEditor.SaveMacro(macroEditor)
	local macroName = XMacros.Utils.Trim(macroEditor.nameText:GetText())
	local macro = XMacros.Utils.Trim(macroEditor.editText:GetText())
	
	if string.len(macro) < 1 then
		XMacros.MacroEditor.SetEnabled(macroEditor, false)
		XMacros.WarningWindow.ShowWarningWindow(macroEditor.wrnWindow, "Cannot save macro with a blank name", function() XMacros.MacroEditor.SetEnabled(macroEditor, true) end)
	elseif string.len(macroName) < 1 then
		XMacros.MacroEditor.SetEnabled(macroEditor, false)
		XMacros.WarningWindow.ShowWarningWindow(macroEditor.wrnWindow, "Cannot save a blank macro", function() XMacros.MacroEditor.SetEnabled(macroEditor, true) end)
	else
		if macroName ~= macroEditor.macroName then
			local oldMacro = XMacros.MacroManager.GetMacro(macroEditor.manager, macroEditor.category, macroName, true)
			if oldMacro then
				XMacros.MacroEditor.SetEnabled(macroEditor, false)
				XMacros.ConfirmationWindow.ShowConfirmationWindow(macroEditor.cfmWindow, "Do you want to overwrite macro " .. macroName .. "?", function(a, okPressed) XMacros.MacroEditor.SaveMacroCallback(macroEditor, macroName, macro, okPressed) end)
				return
			end
		end
		
		XMacros.MacroEditor.SaveMacroCallback(macroEditor, macroName, macro, true)
	end
end

function XMacros.MacroEditor.RedrawMacroList(macroEditor)
	macroEditor.prevButton:SetEnabled(macroEditor.page ~= 1)
	macroEditor.nextButton:SetEnabled(macroEditor.page ~= macroEditor.maxPage)
	
	local rows = XMacros.RowLayout.GetNumRows(macroEditor.rowLayout)
	for row = 1, rows do
		local txtFrame = XMacros.RowLayout.GetCellFrame(macroEditor.rowLayout, row, 1)
		local textIndex = (rows * (macroEditor.page - 1)) + row
		if textIndex <= #macroEditor.macroNames then
			txtFrame:SetText(macroEditor.macroNames[textIndex])
			txtFrame.Event.LeftClick = function() XMacros.MacroEditor.ChooseText(macroEditor, textIndex) end
			txtFrame:SetVisible(true)
		else
			txtFrame:SetVisible(false)
			txtFrame.Event.LeftClick = function() end
		end
	end
end

function XMacros.MacroEditor.ChooseText(macroEditor, textIndex)
	local macro = XMacros.MacroManager.GetMacro(macroEditor.manager, macroEditor.category, macroEditor.macroNames[textIndex], true)
	if macro then
		macro = XMacros.Utils.Trim(macro)
		macroEditor.macroName = macroEditor.macroNames[textIndex]
		macroEditor.editText:SetText(macro)
		macroEditor.macro = macro
		macroEditor.editText:SetCursor(0)
		macroEditor.nameText:SetText(macroEditor.macroName)
	end
end

function XMacros.MacroEditor.PageTurn(macroEditor, pageIncrement)
	macroEditor.page = macroEditor.page + pageIncrement
	if macroEditor.page < 1 then
		macroEditor.page = 1
	end
	if macroEditor.page > macroEditor.maxPage then
		macroEditor.page = macroEditor.maxPage
	end
	
	XMacros.MacroEditor.RedrawMacroList(macroEditor)
end
