-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.MacroManager = {}
XMacros.MacroManager.DefaultCategory = "Default"
XMacros.MacroManager.AbilityCategory = "Ability"
XMacros.MacroManager.PlayerToken = "{Player}"
XMacros.MacroManager.PlayerName = nil

function XMacros.MacroManager.Create(macroDB)
	local manager = {}
	
	manager.macroDB = macroDB
	if not manager.macroDB then
		manager.macroDB = {}
	end
	
	if not manager.macroDB[XMacros.MacroManager.DefaultCategory] then
		manager.macroDB[XMacros.MacroManager.DefaultCategory] = {}
	end
			
	return manager
end

function XMacros.MacroManager.GetMacros(manager)
	return manager.macroDB
end

function XMacros.MacroManager.GetCategoryNames(manager)
	local catNames = {}
	for name, _ in pairs(manager.macroDB) do
		table.insert(catNames, name)
	end
	
	table.insert(catNames, name)
	
	table.sort(catNames)
	
	return catNames
end

function XMacros.MacroManager.GetMacroNames(manager, category)
	if category == XMacros.MacroManager.AbilityCategory then
		return XMacros.AbilityManager.GetAbilityNames()
	else
		local macroNames = {}
	
		local macros = manager.macroDB[category]
		if macros then
			for name, _ in pairs(macros) do
				table.insert(macroNames, name)
			end

			table.sort(macroNames)
		end
	
		return macroNames
	end
end

function XMacros.MacroManager.GetCategory(manager, category)
	if category == XMacros.MacroManager.AbilityCategory then
		return nil
	end
	
	local macros = manager.macroDB
	if macros then
		local catStr = category
		if not category or string.len(tostring(category)) < 1 then
			catStr = XMacros.MacroManager.DefaultCategory
		end
		
		local cat = macros[catStr]
		if not cat then
			macros[catStr] = {}
			cat = macros[catStr]
		end

		return cat
	end
end

function XMacros.MacroManager.DeleteCategory(manager, category)
	local macros = manager.macroDB
	if macros then
		if category and category ~= XMacros.MacroManager.DefaultCategory then
			macros[category] = nil
		end
	end
end

function XMacros.MacroManager.GetMacro(manager, category, name, dontExpand)
	if category == XMacros.MacroManager.AbilityCategory then
		return "cast " .. name
	else
		local cat = XMacros.MacroManager.GetCategory(manager, category)
		if cat then
			local macro = cat[name]
			if macro and dontExpand ~= true then
				return XMacros.MacroManager.ExpandTokens(manager, macro)
			end

			return macro
		end
	end
end

function XMacros.MacroManager.AddMacro(manager, category, name, macro)
	local macros = manager.macroDB
	if macros then
		local cat = XMacros.MacroManager.GetCategory(manager, category)
		if cat then
			cat[name] = macro
		end
	end
end

function XMacros.MacroManager.DeleteMacro(manager, category, name)
	local macros = manager.macroDB
	if macros then
		local cat = XMacros.MacroManager.GetCategory(manager, category)
		if cat then
			cat[name] = nil
		end
	end
end

function XMacros.MacroManager.FindMacros(manager, searchString)
	local foundMacros = {}
	
	for catName, category in pairs(manager.macroDB) do
		for macroName, macro in pairs(category) do
			if string.find(macro, searchString) ~= nil then
				foundMacros[catName][macroName] = 1
			end
		end
	end
	
	return foundMacros
end

function XMacros.MacroManager.GetPlayerName()
	if not XMacros.MacroManager.PlayerName then
		local playerDetails = Inspect.Unit.Detail("player")
		if playerDetails then
			XMacros.MacroManager.PlayerName = playerDetails.name
		else
			return "Unknown"
		end
	end
	
	return XMacros.MacroManager.PlayerName
end

function XMacros.MacroManager.ExpandTokens(manager, macro)
	local lineFeedMacro = string.gsub(macro, "\13", "\10")
	local pos = string.find(lineFeedMacro, XMacros.MacroManager.PlayerToken)
	if pos then
		local playerName = XMacros.MacroManager.GetPlayerName()		
		return string.gsub(lineFeedMacro, XMacros.MacroManager.PlayerToken, playerName)
	end

	return lineFeedMacro
end
