-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.TextPicker = {}
XMacros.TextPicker.name = "XMacros.TextPicker"

--
-- Main functions
--

--
-- UI functions
--

function XMacros.TextPicker.Create(parentFrame, positionFrame, multiSelect)
	local txtPicker = {}
	txtPicker.parentFrame = parentFrame
	txtPicker.multiSelect = multiSelect
	txtPicker.page = 1
	txtPicker.maxPage = 1
	txtPicker.textList = {}
	
	txtPicker.width = 400
	txtPicker.height = 628
	txtPicker.textFrame = UI.CreateFrame("RiftWindow", "TextFrame", parentFrame)
	txtPicker.textFrame:SetPoint("CENTER", positionFrame, "CENTER")
	txtPicker.textFrame:SetWidth(txtPicker.width)
	txtPicker.textFrame:SetHeight(txtPicker.height)
	txtPicker.textFrame:SetVisible(false)
	txtPicker.textFrame:SetLayer(parentFrame:GetLayer() + 1)
	txtPicker.x = txtPicker.textFrame:GetLeft()
	txtPicker.y = txtPicker.textFrame:GetTop()

	local xMargin = 20
	local yMargin = 20
	txtPicker.okButton = UI.CreateFrame("RiftButton", "OKButton", txtPicker.textFrame)
	txtPicker.okButton:SetPoint("BOTTOMLEFT", txtPicker.textFrame, "BOTTOMLEFT", xMargin, -yMargin)
	txtPicker.okButton:SetText("OK")
	txtPicker.okButton.Event.LeftPress = function() XMacros.TextPicker.WindowClose(txtPicker, true) end	

	txtPicker.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtPicker.textFrame)
	txtPicker.cancelButton:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	txtPicker.cancelButton:SetText("Cancel")
	txtPicker.cancelButton.Event.LeftPress = function() XMacros.TextPicker.WindowClose(txtPicker, false) end	

	local pathMargin = -yMargin
	if txtPicker.multiSelect == true then
		pathMargin = 0
	end

	txtPicker.pathLabel = UI.CreateFrame("Text", "PathLabel", txtPicker.textFrame)
	txtPicker.pathLabel:SetPoint("BOTTOMLEFT", txtPicker.okButton, "TOPLEFT", 0, pathMargin)
	txtPicker.pathLabel:SetText("Value:")

	txtPicker.pathText = UI.CreateFrame("RiftTextfield", "PathText", txtPicker.textFrame)
	txtPicker.pathText:SetPoint("BOTTOMRIGHT", txtPicker.cancelButton, "TOPRIGHT", 0, pathMargin)
	txtPicker.pathText:SetPoint("BOTTOMLEFT", txtPicker.pathLabel, "BOTTOMRIGHT", xMargin, 0)
	txtPicker.pathText:SetPoint("TOPLEFT", txtPicker.pathLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.pathText:SetText("")
	txtPicker.pathText:SetBackgroundColor(0.15, 0.15, 0.15, 0.4)

	local frameMiddle = math.floor(txtPicker.textFrame:GetWidth() / 2)
	txtPicker.prevButton = UI.CreateFrame("RiftButton", "PrevButton", txtPicker.textFrame)
	txtPicker.prevButton:SetPoint("BOTTOMRIGHT", txtPicker.pathLabel, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)
	txtPicker.prevButton:SetText("<<")
	txtPicker.prevButton.Event.LeftPress = function() XMacros.TextPicker.PageTurn(txtPicker, -1) end	

	txtPicker.nextButton = UI.CreateFrame("RiftButton", "NextButton", txtPicker.textFrame)
	txtPicker.nextButton:SetPoint("BOTTOMLEFT", txtPicker.pathText, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.nextButton:SetText(">>")
	txtPicker.nextButton.Event.LeftPress = function() XMacros.TextPicker.PageTurn(txtPicker, 1) end
	
	txtPicker.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtPicker.textFrame)
	txtPicker.gridFrame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT", xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("TOPRIGHT", txtPicker.textFrame, "TOPRIGHT", -xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("BOTTOMLEFT", txtPicker.prevButton, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.gridFrame:SetPoint("BOTTOMRIGHT", txtPicker.nextButton, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)

	local margin = 4
	local textSize = txtPicker.pathLabel:GetHeight()
	local rows = math.floor((txtPicker.gridFrame:GetHeight() + margin) / (textSize + margin))
	local cols = 1
	local colWidth = txtPicker.textFrame:GetWidth() - (2 * xMargin)
	if txtPicker.multiSelect == true then
		cols = 2
		colWidth = colWidth - 32 - margin
	end
	txtPicker.rowLayout = XMacros.RowLayout.Create(txtPicker.gridFrame, rows, cols, textSize, colWidth, margin, margin)
	for row = 1, rows do
		local txtFrame = XMacros.RowLayout.CreateFrame(txtPicker.rowLayout, row, cols, "Text")
		if row % 2 == 1 then
			txtFrame:SetBackgroundColor(0, 0, 0.15)
		else
			txtFrame:SetBackgroundColor(0.15, 0.15, 0.15)
		end
		
		if txtPicker.multiSelect == true then
			local checkFrame = XMacros.RowLayout.CreateFrame(txtPicker.rowLayout, row, 1, "RiftCheckbox", XMacros.RowLayout.centerFrame)
		end
	end

	if txtPicker.multiSelect == true then
		XMacros.RowLayout.SetColWidth(txtPicker.rowLayout, 1, 32)
		txtPicker.pathLabel:SetVisible(false)
		txtPicker.pathText:SetVisible(false)
		txtPicker.pathLabel:SetVisible(false)
		txtPicker.pathText:SetVisible(false)
	end
	
	XMacros.TextPicker.Redraw(txtPicker)
	XMacros.RowLayout.Redraw(txtPicker.rowLayout)

	XMacros.TextPicker.SetDragEnabled(txtPicker, true)

	return txtPicker
end

function XMacros.TextPicker.Show(txtPicker, title, textList, callback, checkList)
	txtPicker.callback = callback
	txtPicker.textList = textList
	txtPicker.textFrame:SetTitle(title)
	if txtPicker.multiSelect == true then
		if checkList and #checkList == #textList then
			txtPicker.checkList = checkList
		else
			txtPicker.checkList = {}
			for i = 1, #textList do
				table.insert(txtPicker.checkList, false)
			end
		end
	end
	txtPicker.page = 1
	txtPicker.maxPage = math.ceil(#textList / XMacros.RowLayout.GetNumRows(txtPicker.rowLayout))
	txtPicker.pathText:SetText("")
	XMacros.TextPicker.Redraw(txtPicker)
	txtPicker.textFrame:SetVisible(true)
	txtPicker.x = txtPicker.textFrame:GetLeft()
	txtPicker.y = txtPicker.textFrame:GetTop()
end

function XMacros.TextPicker.WindowClose(txtPicker, okPressed)
	txtPicker.pathText:SetKeyFocus(false)
	txtPicker.textFrame:SetVisible(false)
	
	if okPressed then
		if txtPicker.multiSelect == true then
			txtPicker.callback(txtPicker, txtPicker.checkList)
		else
			local text = XMacros.Utils.Trim(txtPicker.pathText:GetText())
			if text and string.len(text) > 0 then
				txtPicker.callback(txtPicker, text)
			else
				txtPicker.callback(txtPicker)
			end
		end		
	else
		txtPicker.callback(txtPicker)
	end
end

function XMacros.TextPicker.Redraw(txtPicker)
	txtPicker.prevButton:SetEnabled(txtPicker.page ~= 1)
	txtPicker.nextButton:SetEnabled(txtPicker.page ~= txtPicker.maxPage)
	
	local rows = XMacros.RowLayout.GetNumRows(txtPicker.rowLayout)
	local cols = XMacros.RowLayout.GetNumCols(txtPicker.rowLayout)
	for row = 1, rows do
		local chkFrame = nil
		if txtPicker.multiSelect == true then
			chkFrame = XMacros.RowLayout.GetCellFrame(txtPicker.rowLayout, row, 1)	
		end
		
		local txtFrame = XMacros.RowLayout.GetCellFrame(txtPicker.rowLayout, row, cols)
		local textIndex = (rows * (txtPicker.page - 1)) + row
		if textIndex <= #txtPicker.textList then
			txtFrame:SetText(txtPicker.textList[textIndex])
			txtFrame.Event.LeftClick = function() XMacros.TextPicker.ChooseText(txtPicker, textIndex) end
			txtFrame:SetVisible(true)
			if txtPicker.multiSelect == true then
				txtFrame.Event.LeftClick = function() XMacros.TextPicker.TextCheckChanged(txtPicker, textIndex, chkFrame) end
				chkFrame.Event.CheckboxChange = function() end
				chkFrame:SetChecked(txtPicker.checkList[textIndex])
				chkFrame.Event.CheckboxChange = function() XMacros.TextPicker.CheckChanged(txtPicker, textIndex, chkFrame:GetChecked()) end
				chkFrame:SetVisible(true)
			end
		else
			txtFrame:SetVisible(false)
			txtFrame.Event.LeftClick = function() end
			if txtPicker.multiSelect == true then
				chkFrame:SetVisible(false)
				chkFrame.Event.CheckboxChange = function() end
			end
		end
	end
end

function XMacros.TextPicker.TextCheckChanged(txtPicker, textIndex, chkFrame)
	chkFrame:SetChecked(not chkFrame:GetChecked())
end

function XMacros.TextPicker.CheckChanged(txtPicker, textIndex, checked)
	txtPicker.checkList[textIndex] = checked
end

function XMacros.TextPicker.ChooseText(txtPicker, textIndex)
	txtPicker.pathText:SetText(txtPicker.textList[textIndex])
end

function XMacros.TextPicker.PageTurn(txtPicker, pageIncrement)
	txtPicker.page = txtPicker.page + pageIncrement
	if txtPicker.page < 1 then
		txtPicker.page = 1
	end
	if txtPicker.page > txtPicker.maxPage then
		txtPicker.page = txtPicker.maxPage
	end
	
	XMacros.TextPicker.Redraw(txtPicker)
end

function XMacros.TextPicker.SetDragEnabled(txtPicker, enabled)
	if enabled == false then
		if txtPicker.dragFrame then
			txtPicker.dragFrame.frame.Event.RightUp = nil
			txtPicker.dragFrame.frame.Event.RightUpoutside = nil
			txtPicker.dragFrame.frame.Event.RightDown = nil
			txtPicker.dragFrame.frame.Event.MouseMove = nil
			txtPicker.dragFrame.frame = nil
			txtPicker.dragFrame = nil
		end
	elseif not txtPicker.dragFrame then
		local dragFrame = {}

		dragFrame.dragging = false
		dragFrame.frame = UI.CreateFrame("Frame", "DragFrame", txtPicker.textFrame)
		dragFrame.frame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT")
		dragFrame.frame:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "TOPRIGHT", 0, 50)
		dragFrame.frame:SetLayer(txtPicker.textFrame:GetLayer() + 1)
		dragFrame.frame:SetMouseMasking("full")
		dragFrame.frame.Event.RightUp = function () XMacros.TextPicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.RightUpoutside = function () XMacros.TextPicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.RightDown = function () XMacros.TextPicker.MouseDown(txtPicker) end
		dragFrame.frame.Event.LeftUp = function () XMacros.TextPicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.LeftUpoutside = function () XMacros.TextPicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.LeftDown = function () XMacros.TextPicker.MouseDown(txtPicker) end
		dragFrame.frame.Event.MouseMove = function (event, x, y) XMacros.TextPicker.MouseMove(txtPicker, x, y) end
		txtPicker.dragFrame = dragFrame
	end
end

function XMacros.TextPicker.MouseDown(txtPicker)
	if txtPicker.dragFrame.dragging == false then
		local mouse = Inspect.Mouse()
		txtPicker.dragFrame.xOffset = txtPicker.x - mouse.x
		txtPicker.dragFrame.yOffset = txtPicker.y - mouse.y
		txtPicker.dragFrame.dragging = true
	end
end

function XMacros.TextPicker.MouseMove(txtPicker, x, y)
	if txtPicker.dragFrame.dragging == true then
		local totalWidth = UIParent:GetWidth()
		local totalHeight = UIParent:GetHeight()
		txtPicker.x = x + txtPicker.dragFrame.xOffset
		txtPicker.y = y + txtPicker.dragFrame.yOffset
		if txtPicker.x < 0 then txtPicker.x = 0 end
		if txtPicker.y < 0 then txtPicker.y = 0 end
		if txtPicker.x + txtPicker.width > totalWidth then txtPicker.x = totalWidth - txtPicker.width end
		if txtPicker.y + txtPicker.height > totalHeight then txtPicker.y = totalHeight - txtPicker.height end
		txtPicker.textFrame:ClearPoint("CENTER")
		txtPicker.textFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", txtPicker.x, txtPicker.y)
	end
end

function XMacros.TextPicker.MouseUp(txtPicker)
	txtPicker.dragFrame.dragging = false
end


